#include "wx/wx.h"
#include "DualTreeBase.h"

const INT_8 DualTreeBASE_ITEM::ITEM_KIND_NODE = 1;
const INT_8 DualTreeBASE_ITEM::ITEM_KIND_DATA = 2;

const long DualTreeBASE_ItemNODE::EXTENT_Capacity_CPArry = 4 ;

extern bool g_isCASE_SENSITIVE_FileName ;
//   'g_isCASE_SENSITIVE_FileName' is defined in MyCommon_Win.cpp

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ITEM::DualTreeBASE_ITEM( int kind, const DualTreeBASE_BODY * pBody )
{
    m_Item_Kind  = kind ;

    m_pMyBody = (DualTreeBASE_BODY *) pBody ;

    Init_Base( 0, 0, 0, wxT("") );
}

//*****************************************************************************
//
//*****************************************************************************

//?-1/DualTreeBASE_ITEM::DualTreeBASE_ITEM
//?-1/(
//?-1/    ushort kind,
//?-1/    DualTreeBASE_ItemNODE * pMother, 
//?-1/    int   which,
//?-1/    long  sibl_Order,
//?-1/    const wxString & name
//?-1/)
//?-1/{
//?-1/    Init( kind, pMother, which, sibl_Order, name );
//?-1/}

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ITEM::~DualTreeBASE_ITEM()
{
    m_pCustomData = 0 ;
    m_pMyBody     = 0 ;
    //::wxLogDebug( wxT("DualTreeBASE_ITEM::~DualTreeBASE_ITEM() this = %p"), this ) ;
}

//*****************************************************************************
//
//*****************************************************************************

void
DualTreeBASE_ITEM::Init_Base
(
    DualTreeBASE_ItemNODE * pMother,
    int   which,       //=0,1 --> either-side,  =2 --> both-side
    long  sibl_Order,
    const wxString & name
)
{
    m_Name_L[ 0 ] = wxT("") ;
    m_Name_L[ 1 ] = wxT("") ;

    m_Is_Valid_L[ 0 ] = 0;
    m_Is_Valid_L[ 1 ] = 0;
    m_pMother_L [ 0 ] = 0 ;
    m_pMother_L [ 1 ] = 0 ;

    m_Sibl_Order = sibl_Order ;

    if( pMother )
    {
        m_pMyBody = pMother->m_pMyBody ;
    }

    if( which < 0 )
    {
        ;//nothing to do
    }
    else if( which >= 2 )
    {
        m_Name_L    [ 0 ] = name ;
        m_pMother_L [ 0 ] = pMother ;
        m_Is_Valid_L[ 0 ] = 1;

        m_Name_L    [ 1 ] = name ;
        m_pMother_L [ 1 ] = pMother ;
        m_Is_Valid_L[ 1 ] = 1;
    }
    else
    {
        m_Name_L    [ which ] = name ;
        m_pMother_L [ which ] = pMother ;
        m_Is_Valid_L[ which ] = 1;
    }

}

//*****************************************************************************
//
//*****************************************************************************

int
DualTreeBASE_ITEM::Is_Node() const
{
    return( m_Item_Kind == DualTreeBASE_ITEM::ITEM_KIND_NODE ) ;
}

//*****************************************************************************
//
//*****************************************************************************

int
DualTreeBASE_ITEM::Depth() const
{
    int  dep = 0 ;

    if( m_pMother_L[ 0 ] )
    {
        dep += m_pMother_L[ 0 ]->Depth() ;
    }
    else if( m_pMother_L[ 1 ] )
    {
        dep += m_pMother_L[ 1 ]->Depth() ;
    }

    return( dep ) ;
}

//#############################################################################

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ItemNODE::DualTreeBASE_ItemNODE( const DualTreeBASE_BODY * pBody )
      : DualTreeBASE_ITEM( ITEM_KIND_NODE, pBody )
//?-1/      : DualTreeBASE_ITEM( ITEM_KIND_NODE, 0, 2, 0, wxT("") )
{
    Init_Base( 0, 2, 0, wxT("") ) ;

    m_ChildPtr_Arry   = 0 ;
    m_Capacity_CPArry = 4 ;
    m_Count_Children  = 0 ;
    m_State_Load_L[0] = 0 ;
    m_State_Load_L[1] = 0 ;
    m_ChildPtr_Arry = (DualTreeBASE_ITEM **) 
                         new myDualTree_ItemBasePTR [ m_Capacity_CPArry ] ;
}

//*****************************************************************************
//
//*****************************************************************************

//?-1/DualTreeBASE_ItemNODE::DualTreeBASE_ItemNODE
//?-1/(
//?-1/    DualTreeBASE_ItemNODE * pMother,
//?-1/    int which, 
//?-1/    long  sibl_Order,
//?-1/    const wxString & name
//?-1/)  : DualTreeBASE_ITEM( ITEM_KIND_NODE, pMother, which, sibl_Order, name )
//?-1/{
//?-1/    m_ChildPtr_Arry   = 0 ;
//?-1/    m_Capacity_CPArry = 4 ;
//?-1/    m_Count_Children  = 0 ;
//?-1/    m_State_Load_L[0] = 0 ;
//?-1/    m_State_Load_L[1] = 0 ;

//?-1/    m_ChildPtr_Arry = (DualTreeBASE_ITEM **) 
//?-1/                         new myDualTree_ItemBasePTR [ m_Capacity_CPArry ] ;
//?-1/}

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ItemNODE::~DualTreeBASE_ItemNODE()
{
    Clear_Children() ;

    if( m_ChildPtr_Arry )
        delete[] m_ChildPtr_Arry ;
}

//*****************************************************************************
// load state Coherency
// AY side(=which)  invalid ȂAY sidéALoad ς݂Ƃ݂Ȃ
// łȂ(valid)ȂAm_State_Load_L[ which ] ̒lɏ]
//*****************************************************************************

int
DualTreeBASE_ItemNODE::State_Load_Coh
(
    int which     //Y side id.  = 0 --> source side, 
                  //                 = 1 --> destination side
)  const
{
    if( m_Is_Valid_L[ which ] == 0 )  return( 1 ) ;

    return( m_State_Load_L[ which ] ) ;
}

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ITEM *
DualTreeBASE_ItemNODE::Item_Ptr
(
    long  index   // = 0,1,2,3,...
)  const
{
    if( index >= m_Count_Children )
        return( 0 ) ;
    return  m_ChildPtr_Arry[ index ] ;
}

//*****************************************************************************
//
//*****************************************************************************

long
DualTreeBASE_ItemNODE::Cnt_Children() const
{
    return( m_Count_Children ) ;
}

//*****************************************************************************
//
//*****************************************************************************

long
DualTreeBASE_ItemNODE::Get_Index
( 
    DualTreeBASE_ITEM * pChildItem 
) const
{
    long idx ;

    for( idx = 0 ; ; idx ++ )
    {
        if( idx >= m_Count_Children )  return( -1 ) ;

        if( pChildItem == m_ChildPtr_Arry[ idx ] ) break ;
    }

    return( idx ) ;
}

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ITEM * 
DualTreeBASE_ItemNODE::Search_Item_ByName
(
    int   is_Node,
    int   which, 
    const wxString & name_New
) const
{
    extern bool Boolian_Eq( int p1, int p2 );

    const DualTreeBASE_ITEM * pItem ;
    wxString  name = name_New ; 
    wxString  name_wk ; 
    long  idx ;

    if( ! g_isCASE_SENSITIVE_FileName )
        name.MakeUpper() ;

    for( idx = 0 ; ; idx ++ )
    {
        if( idx >= m_Count_Children )  return( 0 ) ;

        pItem = m_ChildPtr_Arry[ idx ] ;

        if( ! Boolian_Eq( is_Node, pItem->Is_Node() ) )  continue ;

        if( pItem->m_Is_Valid_L[ which ] == 0 ) continue ;

        name_wk = pItem->m_Name_L[ which ] ;

        if( ! g_isCASE_SENSITIVE_FileName )
            name_wk.MakeUpper() ;

        if( name_wk == name ) break ;
    }

    return( (DualTreeBASE_ITEM *) pItem ) ;
}

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ITEM * 
DualTreeBASE_ItemNODE::Search_Item_ByName_Both
(
    int   is_Node,              //==0 --> Data`item TA<> 0 --> Node T
    const wxString & name_New   //TׂO
) const
{
    extern bool Boolian_Eq( int p1, int p2 );

    const DualTreeBASE_ITEM * pItem ;
    wxString  name = name_New ; 
    wxString  name_wk ; 
    long  idx ;

    if( ! g_isCASE_SENSITIVE_FileName )
        name.MakeUpper() ;

    for( idx = 0 ; ; idx ++ )
    {
        if( idx >= m_Count_Children )  return( 0 ) ;

        pItem = m_ChildPtr_Arry[ idx ] ;

        if( ! Boolian_Eq( is_Node, pItem->Is_Node() ) )  continue ;

        if( pItem->m_Is_Valid_L[ 0 ] )
        {
            name_wk = pItem->m_Name_L[ 0 ] ;

            if( ! g_isCASE_SENSITIVE_FileName )
                name_wk.MakeUpper() ;

            if( name_wk == name ) break ;
        }

        if( pItem->m_Is_Valid_L[ 1 ] )
        {
            name_wk = pItem->m_Name_L[ 1 ] ;

            if( ! g_isCASE_SENSITIVE_FileName )
                name_wk.MakeUpper() ;

            if( name_wk == name ) break ;
        }
    }

    return( (DualTreeBASE_ITEM *) pItem ) ;
}

//*****************************************************************************
//  w index  item ̌ pItem_New }
//*****************************************************************************

int
DualTreeBASE_ItemNODE::Insert_New_Child
(
    DualTreeBASE_ITEM * pItem_New,
    long   idx
)
{
    DualTreeBASE_ITEM ** old_childPtr_Arry ;

    if( m_Count_Children >= m_Capacity_CPArry )
    {
        old_childPtr_Arry  = m_ChildPtr_Arry ;
        m_Capacity_CPArry += EXTENT_Capacity_CPArry ;
        m_ChildPtr_Arry    = (DualTreeBASE_ITEM **) new myDualTree_ItemBasePTR [ m_Capacity_CPArry ] ;
        if( m_ChildPtr_Arry == 0 )  goto Err_Fatal ;

        memcpy( m_ChildPtr_Arry, old_childPtr_Arry, sizeof( m_ChildPtr_Arry ) * m_Count_Children ) ;
        delete[]  old_childPtr_Arry ;
    }

    if( idx < 0 )
        idx = m_Count_Children ;

    memmove( & m_ChildPtr_Arry[ idx + 1 ], & m_ChildPtr_Arry[ idx ],
                sizeof( m_ChildPtr_Arry ) * (m_Count_Children - idx ) ) ;
    m_Count_Children ++ ;
    m_ChildPtr_Arry[ idx ] = pItem_New ;

    return( 0 ) ;

 Err_Fatal :
        ::wxLogMessage( wxT("DualTreeBASE_ItemNODE::Insert_New_Child() Fatal error") );

    *m_ChildPtr_Arry = 0 ; //to invoke memory violation
    return( -1 ) ;
}

//*****************************************************************************
// A, B ꂩSide  Item ǉ
//
//    result = 0 ;  side  OItem 
//    result = 1 ; VK child-Node 
//    result = 2 ;  side ɊɓO݂Ă: (Error Ƃ͌Ȃ)
//    result = -1 ; Node Ȃ  system memory insufficient
//    result = -2 ; m_pMyBody ȂB fatal logical error
//*****************************************************************************

int
DualTreeBASE_ItemNODE::Add_Child_Item
(
    int  is_Node,
    int  which,           //= 0 : A-Side
                          //= 1 : B-Side
    const wxString & name,
    DualTreeBASE_ITEM ** ppItem,    //<out>
    long idx 
)
{
    DualTreeBASE_ITEM * pItem = 0 ;
    int  other, result ;

    result = -2 ;
    if( m_pMyBody == 0 )  goto  EXIT ;

    result = 0 ;

    pItem = Search_Item_ByName( is_Node, which, name ) ;
        //::wxLogDebug( wxT("DualTreeBASE_ItemNODE::Add_Child_Item() isNode= %d, which =%d, nam=<%s>, pItem= $%p"), 
        //                                is_Node, which, name, pItem );
    if( pItem )
    {
        result = 2 ;// side ɊɓO݂Ă

        m_pMyBody->Set_Item( pItem, which, 1 ) ;//Time Stamp, size Đݒ
        goto  EXIT ;
    }

    other = (which + 1) % 2 ;
    pItem = Search_Item_ByName( is_Node, other, name ) ;

    if( pItem  )
    {       // side  OItem 
        pItem->m_Name_L   [ which ] = name ;
        pItem->m_pMother_L[ which ] = this ;

        m_pMyBody->Set_Item( pItem, which, 1 ) ;

        result = 0 ;
        goto EXIT ;
    }

    pItem = (DualTreeBASE_ITEM *) m_pMyBody->Create_Item( is_Node, this,
                                                             which, name ) ;
                //::wxLogDebug( wxT("DualTreeBASE_ItemNODE::Add_Child_Item: Create: pItem=%p"),
                //                    pItem );
    if( pItem == 0 )
    {
        result = -1 ;
        goto  EXIT ;
    }

    result = 1 ;

    Insert_New_Child( pItem, idx ) ;
 
 EXIT :

    if( ppItem )
        *ppItem = pItem ;
                    //::wxLogDebug( wxT("DualTreeBASE_ItemNODE::Add_Child_Item() : ---> result=%d"), result );
    return( result ) ;
}

//*****************************************************************************
// A, B both-side  item ǉ. (eAO)
//    result = 1 ; VK child-Node 
//    result = 2 ; ǂ炩side ܂͗sideɊɓO݂Ă: (Error Ƃ͌Ȃ)
//                  Е̂ݑݏꍇ́AS鎖
//    result = -1 ; Node Ȃ  system memory insufficient
//    result = -2 ; m_pMyBody ȂB fatal logical error
//*****************************************************************************

int
DualTreeBASE_ItemNODE::Add_Child_Item_Both
(
    int  is_Node,
    const wxString & name,
    DualTreeBASE_ITEM ** ppItem,    //<out>
    long idx 
)
{
    DualTreeBASE_ITEM * pItem = 0 ;
    int  result ;

    result = -2 ;
    if( m_pMyBody == 0 )  goto  EXIT ;

    result = 0 ;

    pItem = Search_Item_ByName_Both( is_Node, name ) ;
        //::wxLogDebug( wxT("DualTreeBASE_ItemNODE::Add_Child_Item_Both() isNode= %d, nam=<%s>, pItem= $%p"), 
        //                                is_Node, name, pItem );
    if( pItem )
    {
        result = 2 ;//ǂ炩side ܂͗ɊɓO݂Ă

        if( pItem->m_Is_Valid_L[0] == 0 )
        {
            pItem->m_pMother_L[ 0 ] = this ;
            pItem->m_Name_L   [ 0 ] = name ;
            m_pMyBody->Set_Item( pItem, 0, 1 ) ;//Time Stamp, size Đݒ
        }

        if( pItem->m_Is_Valid_L[1] == 0 )
        {
            pItem->m_pMother_L[ 1 ] = this ;
            pItem->m_Name_L   [ 1 ] = name ;
            m_pMyBody->Set_Item( pItem, 1, 1 ) ;//Time Stamp, size Đݒ
        }

        goto  EXIT ;
    }

    pItem = m_pMyBody->Create_Item( is_Node, this, 2, name ) ;
                //::wxLogDebug( wxT("DualTreeBASE_ItemNODE::Add_Child_Item_Both: Create: pItem=%p"),
                //                    pItem );
    if( pItem == 0 )
    {
        result = -1 ;
        goto  EXIT ;
    }

    result = 1 ;

    Insert_New_Child( pItem, idx ) ;
 
 EXIT :

    if( ppItem )
        *ppItem = pItem ;
                    //::wxLogDebug( wxT("DualTreeBASE_ItemNODE::Add_Child_Item() : ---> result=%d"), result );
    return( result ) ;
}

//*****************************************************************************
//   dynamic Ȕz  m_ChildPtr_Arry[] size  ŏɍ
//*****************************************************************************

long
DualTreeBASE_ItemNODE::Shrink()
{
    long  capa_Old, capa_New  ;
    DualTreeBASE_ITEM ** childPtr_Arry_new ;
    DualTreeBASE_ITEM * pItem ;
    long  idx, cnt ;

    cnt = 0 ;

    for( idx = 0 ; idx < m_Count_Children ; idx ++ )
    {
        pItem = m_ChildPtr_Arry[ idx ] ;

        if( pItem == 0 )  continue ;

        m_ChildPtr_Arry[ cnt ] = pItem ;
        cnt ++ ;
    }

    m_Count_Children = cnt ;

    if( m_Capacity_CPArry <= m_Count_Children )
        return( m_Capacity_CPArry ) ;

    capa_Old = m_Capacity_CPArry ;

    capa_New = m_Count_Children ;
    if( capa_New <= 0 )
        capa_New = 1 ;

    childPtr_Arry_new  = (DualTreeBASE_ITEM **) new myDualTree_ItemBasePTR [ capa_New ] ;

    memcpy( childPtr_Arry_new, m_ChildPtr_Arry, m_Count_Children * sizeof( myDualTree_ItemBasePTR ) );

    delete[] m_ChildPtr_Arry ;

    m_ChildPtr_Arry = childPtr_Arry_new ;

    m_Capacity_CPArry = capa_New ;

    return( capa_Old ) ;
}

//#############################################################################

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ItemDATA::DualTreeBASE_ItemDATA( const DualTreeBASE_BODY * pBody )
      : DualTreeBASE_ITEM( ITEM_KIND_DATA, pBody )
{
    Init_Base( 0, 2, 0, wxT("") ) ;
}

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ItemDATA::~DualTreeBASE_ItemDATA()
{
    //::wxLogDebug( wxT("DualTreeBASE_ItemDATA::~DualTreeBASE_ItemDATA() this= %p\n<%s> : <%s>"),
    //                        this, m_Name_L[0], m_Name_L[1] ) ;
}

//*****************************************************************************
//  clear all children on the 'which' side
//       
//  return : =  0 ; ̃m[hchildren Sč폜
//           >< 1 : 폜Ȃ̂
//*****************************************************************************

long
DualTreeBASE_ItemNODE::Clear_OneSide( int which )
{
    long  lRet ;

    lRet = Clear_OneSide_R( which ) ;

    m_Is_Valid_L  [ which ] = 0 ;
    m_Name_L      [ which ] = wxT("");
    m_State_Load_L[ which ] = 0 ;

    m_pMyBody->Set_Item( this, which, 0 ) ;

    return( lRet ) ;
};

//*****************************************************************************
//  clear all children on the 'which' side
//  return : =  0 ; ̃m[hchildren Sč폜
//           >< 1 : 폜Ȃ̂
//*****************************************************************************

long
DualTreeBASE_ItemNODE::Clear_OneSide_R( int which )
{
    DualTreeBASE_ITEM * pItem ;
    DualTreeBASE_ItemNODE * pNode ;
    long  idx, cnt, ret ;
    int  other ;
                //::wxLogDebug( wxT("%p: Clear_OneSide_R( %d ): <%s>"), this, which, m_Name_L[which] );

    cnt   = 0 ;
    other = ( which + 1 ) % 2 ;

    for( idx = 0 ; idx < m_Count_Children ; idx ++ )
    {
        pItem = m_ChildPtr_Arry[ idx ] ;

                //::wxLogDebug( wxT("      %p: Clear_OneSide_R(): idx=%d <%s>"), this, idx, pItem->m_Name_L[ which ] );
        if( ! pItem->Is_Node() )
        {
            if( pItem->m_Is_Valid_L[ other ] )
            {         //葤 Node ݂Ă-->Item ͍폜Ȃ
                //={ /100/ side 𖳌
                pItem->m_Is_Valid_L[ which ] = 0 ;
                pItem->m_Name_L    [ which ] = wxT("");

                m_pMyBody->Set_Item( pItem, which, 0 ) ;

                cnt ++ ;
                //}= /100/
            }
            else
            {
                delete ((DualTreeBASE_ItemDATA *)pItem);

                m_ChildPtr_Arry[ idx ] = 0 ;
            }
        }
        else
        {   //child-item  <<Node>> ̏ꍇ

            pNode = (DualTreeBASE_ItemNODE *) pItem ;

            //={ /210/ ܂  Node child-item  clear
            ret = pNode->Clear_OneSide_R( which );

            cnt += ret ;

            if( ret || pItem->m_Is_Valid_L[ other ] )
            {         //葤 child-Node ݂Ă-->Item ͍폜Ȃ
                //={ /211/ side 𖳌
                pNode->m_Is_Valid_L  [ which ] = 0 ;
                pNode->m_Name_L      [ which ] = wxT("");
                pNode->m_State_Load_L[ which ] = 0 ;

                m_pMyBody->Set_Item( pNode, which, 0 ) ;

                cnt ++ ;
                //}= /211/
            }
            else
            {         //葤 child-NodeȂ --> delete child-Node
                delete ((DualTreeBASE_ItemNODE *)pItem);

                m_ChildPtr_Arry[ idx ] = 0 ;
            }
        }
    }

    Shrink() ;
                //::wxLogDebug( wxT("%p: Clear_OneSide_R() Exit"), this );
    return( cnt ) ;
}

//*****************************************************************************
//
//*****************************************************************************

int
DualTreeBASE_ItemNODE::Clear_Children()
{
    DualTreeBASE_ITEM * pItem ;
    long  idx ;

    if( m_ChildPtr_Arry <= 0 )  goto  EXIT ;

    for( idx = 0 ; idx < m_Count_Children ; idx ++ )
    {
        pItem = m_ChildPtr_Arry[ idx ] ;

        if( pItem == 0 )  continue ;

        if( pItem->Is_Node() )
        {
            delete ((DualTreeBASE_ItemNODE *)pItem);
        }
        else
        {
            delete ((DualTreeBASE_ItemDATA *)pItem);
        }

        m_ChildPtr_Arry[ idx ] = 0 ;
    }

 EXIT :

    m_Count_Children = 0 ;

            //::wxLogDebug( wxT("DualTreeBASE_ItemNODE::~DualTreeBASE_ItemNODE():  this= %p, arry=%p\n<%s> : <%s>"),
            //                this, m_ChildPtr_Arry, m_Name_L[0], m_Name_L[1] ) ;
    return( 0 );
}

//*****************************************************************************
//
//*****************************************************************************

int
DualTreeBASE_ItemNODE::Set_Sibling_Order( int is_Recursive )
{
        DualTreeBASE_ITEM * pItem ;
        int   dep, dep_max ;
        long  idx ;

    dep_max = 0 ;

    for( idx = 0 ; idx < m_Count_Children ; idx ++ )
    {
        pItem = m_ChildPtr_Arry[ idx ] ;
        pItem->m_Sibl_Order = idx + 1 ;

        if( is_Recursive == 0 )  continue ;
        if( ! pItem->Is_Node() ) continue ;

        dep_max = 1 ;

        dep = ((DualTreeBASE_ItemNODE *)pItem)->Set_Sibling_Order( 1 ) ;
        if( dep > dep_max )
            dep_max = dep ;
    }

    return( dep_max ) ;
}

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_BODY::DualTreeBASE_BODY()
{
}

//*****************************************************************************
//
//*****************************************************************************

//?-1/DualTreeBASE_BODY::DualTreeBASE_BODY( DualTreeBASE_ItemNODE * pRoot_Node )
//?-1/{
//?-1/    m_pRoot_Node = pRoot_Node ;
//?-1/       //::wxLogDebug( "DualTreeBASE_BODY::DualTreeBASE_BODY() m_Root_Node= %p", m_pRoot_Node );
//?-1/    if( pRoot_Node )
//?-1/        pRoot_Node->m_pMyBody = this ;
//?-1/}

//*****************************************************************************
//
//*****************************************************************************
//virtual
DualTreeBASE_BODY::~DualTreeBASE_BODY()
{
    //::wxLogDebug( "DualTreeBASE_BODY::~DualTreeBASE_BODY() this = %p", this ) ;
}

//*****************************************************************************
//
//*****************************************************************************
//virtual 
int
DualTreeBASE_BODY::Set_Item
( 
    DualTreeBASE_ITEM * pItem, 
    int which, 
    int flag       //side (which) clear 鎞A0
                   //side (which) create 鎞A1 
)
{
    return( 0 ) ;
}

//*****************************************************************************
//
//*****************************************************************************

//virtual 
DualTreeBASE_ITEM * 
DualTreeBASE_BODY::Create_Item
(
    int   is_Node,
    DualTreeBASE_ItemNODE * pMother,
    int   which, 
    const wxString & name
)
{
        DualTreeBASE_ITEM * pItem ;

    if( is_Node )
    {
        pItem = ( DualTreeBASE_ITEM *) new DualTreeBASE_ItemNODE( this );
    }
    else
    {
        pItem = ( DualTreeBASE_ITEM *) new DualTreeBASE_ItemDATA( this );
    }

    pItem->Init_Base( pMother, which, 0, name ) ;

        ::wxLogDebug( wxT("DualTreeBASE_BODY::Create_Item() --> %p\n"), pItem ) ;

    return( pItem ) ;
}

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ITEM * 
DualTreeBASE_ITERATE::First
(
    const DualTreeBASE_ItemNODE * pMother,
    long  index      //=0 means thisself 
)
{
    m_stack_Depth = 0 ;
    m_stack_NodPtr_L[ 0 ] = (DualTreeBASE_ItemNODE *) pMother ;
    m_stack_Index_L [ 0 ] = index   ;

    if( index <= 0 )
        return( (DualTreeBASE_ITEM *) pMother );
    else
        return( pMother->Item_Ptr( index - 1 ) ) ; 
}

//*****************************************************************************
//
//*****************************************************************************

DualTreeBASE_ITEM * 
DualTreeBASE_ITERATE::Next()
{
    DualTreeBASE_ItemNODE * pMother ;
    DualTreeBASE_ITEM * pItem ;
    long idx ;// 

    //(Ô) item AŌ child ł
    //   stack ɖ߂Ă
    for( ; ; )
    {
        idx = m_stack_Index_L[ m_stack_Depth ] ;//
        pMother = m_stack_NodPtr_L[ m_stack_Depth ] ;
            ::wxLogDebug( wxT("DualTreeBASE_ITERATE::Get_Next() :for(): Depth= %hd, idx = %ld, pMother = %p(&ld)"),
                                     m_stack_Depth, idx, pMother, pMother->m_Count_Children );
        if( idx < pMother->m_Count_Children )  break ;//Ō child łȂ

        if( m_stack_Depth <= 0 )     goto  EXIT_Final ;   
        m_stack_Depth -- ;
    }

    idx ++ ;//to next child

    pItem = pMother->Item_Ptr( idx - 1 ) ;

    if( pItem->Is_Node() )
    {
        m_stack_Depth ++ ;
        m_stack_NodPtr_L[ m_stack_Depth ] =(DualTreeBASE_ItemNODE *) pItem ;
        m_stack_Index_L [ m_stack_Depth ] = 0 ;
    }

 EXIT :
        ::wxLogDebug( wxT("  Get_Next() : return: pItem= %p idx=%ld"), pItem, idx ) ;
    return( pItem );

 EXIT_Final :

    pItem = 0 ;
    goto  EXIT ;
}
